      implicit none
      character*(6) whoami
      parameter(whoami='search')
      include "read_shape_mod_size.field_dim.par"
      include "set_b3f.net.par"
      include "set_b3f.nvec.par"
      double precision   b3f(3,nvec,net)
      real               pos(dimpos,npos)
      integer            con(dimcon,ncon)
      real               dat(dimdat,ndat)
      character*(320)    filename
      double precision   lon
      double precision   lat
      double precision   pamin
      double precision   pamax
      double precision   iamin
      double precision   iamax
      double precision   eamin
      double precision   eamax
      character*(160)    name(net)
!  The declarations above have been automatically inserted by my arcs
!  compiler.

!****************************************************************************!
!- Table of contents
!#TOC
!****************************************************************************!
!- 1 Command to compile this program
!- 2 Declarations
!-   2.1 Included parameters
!-   2.2 Parameters
!-   2.3 Variables
!- 3 Read input data
!- 4 Open result file
!- 5 Load Digital Shape Kernel DSK
!-   5.1 Open DSK file
!-   5.2 Get DLA descriptor
!- 6 Get POI in xyz
!- 7 Get surface normal at POI
!- 8 Convert angle limits to rad
!- 9 Loop through B3F
!-   9.1 Check if POI is in FOV
!-       9.1.1 Project POI and bounds in work plane
!-       9.1.2 Loop through edges
!-       9.1.3 If we get here, it's in FOV
!-   9.2 Check illumination angles
!-       9.2.1 Phase angle
!-       9.2.2 Emission angle
!-       9.2.3 Incidence angle
!-       9.2.4 Count OK
!-   9.3 Check if POI is visible
!-   9.4 Write to result file
!-   9.5 Check if POI is illuminated
!- 10 Print results

!****************************************************************************!
!- 1
!# Command to compile this program
!****************************************************************************!
!  gfortran -O2 -o search.x search.f spicelib.a support.a

!****************************************************************************!
!- 2
!# Declarations
!****************************************************************************!

!============================================================================!
!- 2.1
!## Included parameters
!============================================================================!
      include "dla.inc"

!============================================================================!
!- 2.2
!## Parameters
!============================================================================!
      integer            nbound
      parameter        ( nbound = 4 )

!============================================================================!
!- 2.3
!## Variables
!============================================================================!
      double precision   angle
      double precision   bound(3,nbound)
      double precision   bs(3)
      character*(42)     chelp
      double precision   dir(3)
      integer            dladsc(dladsz)
      double precision   ea
      logical            found
      double precision   halfpi
      integer            handle
      integer            i1
      integer            i2
      double precision   ia
      integer            ib
      integer            idim
      integer            iet
      integer            nangleok
      integer            ninfov
      integer            nillum
      integer            nvisible
      integer            hitplid
      double precision   pa
      double precision   poi(3)
      double precision   poinormal(3)
      integer            poiplid
      character*(320)    record
      double precision   rpd
      double precision   scpos(3)
      double precision   sunpos(3)
      double precision   v1(3)
      double precision   v2(3)
      double precision   vdot
      double precision   vnorm
      double precision   vsep

!****************************************************************************!
!- 3
!# Read input data
!****************************************************************************!
!  The commands to read the input data have been automatically inserted by my
!  arcs compiler.
      open( 42,
     &file = 'read_b3f'
     &//'.b3f.d',
     &form = 'UNFORMATTED' )
      read(42) b3f
      close( 42 )
      open( 42,
     &file = 'read_shape_mod'
     &//'.field.d',
     &form = 'UNFORMATTED' )
      read(42) pos
      read(42) con
      read(42) dat
      close( 42 )
      open( 42,
     &file = 'set_dsk'
     &//'.filename.d',
     &form = 'UNFORMATTED' )
      read(42) filename
      close( 42 )
      open( 42,
     &file = 'set_poi'
     &//'.poi.d',
     &form = 'UNFORMATTED' )
      read(42) lon
      read(42) lat
      read(42) pamin
      read(42) pamax
      read(42) iamin
      read(42) iamax
      read(42) eamin
      read(42) eamax
      close( 42 )
      open( 42,
     &file = 'read_navcam_list'
     &//'.product_names.d',
     &form = 'UNFORMATTED' )
      read(42) name
      close( 42 )

!****************************************************************************!
!- 4
!# Open result file
!****************************************************************************!
!      open( 12, file = 'osinac_list_landing_site_visible.txt' )
      open( 12, file = 'outlist.txt' )

!****************************************************************************!
!- 5
!# Load Digital Shape Kernel DSK
!****************************************************************************!

!============================================================================!
!- 5.1
!## Open DSK file
!============================================================================!
      print*, 'Opening DSK '//trim( filename )
      call dasopr( filename, handle )

!============================================================================!
!- 5.2
!## Get DLA descriptor
!============================================================================!
      call dlabfs( handle, dladsc, found )
      if ( .not. found ) stop 'DLA descriptor not found.'

!****************************************************************************!
!- 6
!# Get POI in xyz
!****************************************************************************!
      call latrec( 10d0, lon*rpd(), lat*rpd(), v1 )
      call vscl( -1d0, v1, dir )
      call dskx02( handle, dladsc, v1, dir, poiplid, poi, found )
      if ( found ) then
         print*, 'POI =', ( real( poi(idim) ), idim = 1, 3 )
      else
         stop 'NO INTERSECTION FOUND FOR POI !!!'
      end if

!****************************************************************************!
!- 7
!# Get surface normal at POI
!****************************************************************************!
      do idim = 1, 3
         v1(idim) = pos(idim,con(2,poiplid)+1) -
     &              pos(idim,con(1,poiplid)+1)
         v2(idim) = pos(idim,con(3,poiplid)+1) -
     &              pos(idim,con(1,poiplid)+1)
      end do
      call vcrss( v1, v2, poinormal )
      do idim = 1, 3
         v1(idim) = pos(idim,con(1,poiplid)+1)
      end do
      if ( vdot( v1, poinormal ) .lt. 0d0 ) then
         print*, poiplid
         stop 'NORMAL POINTING INWARD !!!'
      end if

!****************************************************************************!
!- 8
!# Convert angle limits to rad
!****************************************************************************!
      print*, 'Phase angle range    ', real(pamin), real(pamax)
      print*, 'Emission angle range ', real(eamin), real(eamax)
      print*, 'Incidence angle range', real(iamin), real(iamax)
      pamin = pamin * rpd()
      pamax = pamax * rpd()
      eamin = eamin * rpd()
      eamax = eamax * rpd()
      iamin = iamin * rpd()
      iamax = iamax * rpd()

!****************************************************************************!
!- 9
!# Loop through B3F
!****************************************************************************!
!  We first check if the POI is in the FOV, then if some geometrical
!  parameters are within the prescribed limits, and only if all these
!  conditions are fulfilled, we finally check if the POI is visible (not
!  obscured by other terrain), because this last check is by far the
!  most expensive.
      print*, net, 'images'
      ninfov = 0
      nvisible = 0
      nillum = 0
      nangleok = 0
      do iet = 1, net
!         print*, iet

!============================================================================!
!- 9.1
!## Check if POI is in FOV
!============================================================================!
!  We work in a plane which is perpendicular to the boresight vecter and
!  located a unit distance from the SC.

!----------------------------------------------------------------------------!
!- 9.1.1
!### Project POI and bounds in work plane
!----------------------------------------------------------------------------!
         do idim = 1, 3
            scpos(idim) = b3f(idim,1,iet)
            bs(idim) = b3f(idim,3,iet)
!            poi(idim) = b3f(idim,2,iet) ! Sun as POI for testing
         end do
         call vsub( poi, scpos, dir )
         angle = vsep( dir, bs )
         if ( angle .ge. halfpi() ) then
!            ninfov = ninfov + 1 ! for TEST
            goto 4000
         end if
         call vscl( 1d0 / ( cos(angle) * vnorm(dir) ), dir, dir )
!         print*, real( vnorm( dir ) ), ( real( dir(idim) ), idim=1,3 )
         do ib = 1, 4
            do idim = 1, 3
               v1(idim) = b3f(idim,3+ib,iet)
            end do
            angle = vsep( v1, bs )
            call vscl( 1d0 / ( cos(angle) * vnorm(v1) ), v1, v1 )
!            print*, real( vnorm( v1 ) ), ( real( v1(idim) ), idim=1,3 )
            do idim = 1, 3
               bound(idim,ib) = v1(idim)
            end do
         end do

!----------------------------------------------------------------------------!
!- 9.1.2
!### Loop through edges
!----------------------------------------------------------------------------!
!  The approach taken here to check whether the POI is in the FOV only works
!  for convex polygons (thus it would not work for ALICE). There should be
!  some function available which can do such a check for arbitrary polygons.
         do i1 = 1, nbound
            i2 = i1 + 1
            if ( i2 .gt. nbound ) i2 = 1
            do idim = 1, 3
               v1(idim) = dir(idim) - bound(idim,i1)
!               v1(idim) = bs(idim) - bound(idim,i1) ! TEST
               v2(idim) = bound(idim,i2) - bound(idim,i1)
            end do
            call vcrss( v2, v1, v1 )
!            print*
!            print*, bs
!            print*, v1
!            print*, vsep( bs, v1 )
            if ( vsep( bs, v1 ) .ge. halfpi() ) goto 4000
         end do

!----------------------------------------------------------------------------!
!- 9.1.3
!### If we get here, it's in FOV
!----------------------------------------------------------------------------!
         ninfov = ninfov + 1

!============================================================================!
!- 9.2
!## Check illumination angles
!============================================================================!
         do idim = 1, 3
            sunpos(idim) = b3f(idim,2,iet)
         end do

!----------------------------------------------------------------------------!
!- 9.2.1
!### Phase angle
!----------------------------------------------------------------------------!
         pa = vsep( scpos, sunpos )
!         print*, 'pa:', real(pamin), real(pa), real(pamax)
         if ( pa .lt. pamin .or. pa .gt. pamax ) goto 4000

!----------------------------------------------------------------------------!
!- 9.2.2
!### Emission angle
!----------------------------------------------------------------------------!
         ea = vsep( scpos, poinormal )
!         print*, 'ea:', real(eamin), real(ea), real(eamax)
         if ( ea .lt. eamin .or. ea .gt. eamax ) goto 4000

!----------------------------------------------------------------------------!
!- 9.2.3
!### Incidence angle
!----------------------------------------------------------------------------!
         ia = vsep( sunpos, poinormal )
!         print*, 'ia:', real(iamin), real(ia), real(iamax)
         if ( ia .lt. iamin .or. ia .gt. iamax ) goto 4000

!----------------------------------------------------------------------------!
!- 9.2.4
!### Count OK
!----------------------------------------------------------------------------!
         nangleok = nangleok + 1

!============================================================================!
!- 9.3
!## Check if POI is visible
!============================================================================!
         call dskx02( handle, dladsc, scpos, dir, hitplid, v1, found )
         if ( found ) then
            if ( hitplid .eq. poiplid ) then
               nvisible = nvisible + 1
            else
               goto 4000
            end if
         else
            stop 'NO INTERSECTION FOUND FOR VIEW !!!'
         end if

!============================================================================!
!- 9.4
!## Write to result file
!============================================================================!
         write( 12, '(a)' ) trim( name(iet) )
!         chelp = data_set_id(iet)
!         write( 12, '(a,a)' ) chelp, trim( product_id(iet) )

!============================================================================!
!- 9.5
!## Check if POI is illuminated
!============================================================================!
         call vscl( 10d0 / vnorm( sunpos ), sunpos, sunpos )
         call vscl( -1d0, sunpos, dir )
         call vadd( poi, sunpos, sunpos )
         call dskx02( handle, dladsc, sunpos, dir, hitplid, v1, found )
         if ( found ) then
            if ( hitplid .eq. poiplid ) then
               nillum = nillum + 1
            else
               goto 4000
            end if
         else
            stop 'NO INTERSECTION FOUND FOR ILLUMINATION !!!'
         end if

 4000    continue
      end do

!****************************************************************************!
!- 10
!# Print results
!****************************************************************************!
      print*, ninfov, 'of these in FOV'
      print*, nangleok, 'of these with valid illumination angles'
      print*, nvisible, 'of these visible'
      print*, nillum, 'of these illuminated'
      end
